;**************************************************************************
;*
;* FILE  boot.asm
;* Copyright (c) 2005, 2006 Daniel Kahlin <daniel@kahlin.net>
;* Written by Daniel Kahlin <daniel@kahlin.net>
;* $Id: boot.asm,v 1.9 2006-10-20 19:43:01 tlr Exp $
;*
;* DESCRIPTION
;*   Fast C64 DTV server.
;*
;******
	processor 6502

	MAC	sac
	dc.b	$32,{1}
	ENDM
	MAC	sir
	dc.b	$42,{1}
	ENDM
	MAC	bra
	dc.b	$12,{1}-2-.
	ENDM
		
DEFAULT_REG8	equ	%01010101	;all ram

	seg	code
	org	$4000

	rorg	$8000
;**************************************************************************
;*
;* NAME  reset hook ($018000)
;*
;* DESCRIPTION
;*   warm start is used by the normal mem trampolines.
;*
;******
	dc.w	cold_start
	dc.w	warm_start
	dc.b	"D"|$80,"T"|$80,"V"|$80,"8","0"
	dc.w	warm_start

	include	"config.asm"

;**************************************************************************
;*
;* NAME  cold_start, warm_start
;*
;* DESCRIPTION
;*   Entry Point.
;*
;******
cold_start:
; this should already have been done, but just to be safe
	sei
	cld
	ldx	#$ff
	txs

	sir	$ed
; Set $8000-$bfff mapping as $4000-$7fff mapping
	tya	; Acc=reg $e
	tax	; reg $d=Acc
	jmp	cs_continue
	rend
cs_continue:
	ldy	#$02		; Map $008000 -> $8000
	sir	$12

; replicate the cold start at $F91E
	ldx	#$00
	stx	$d020
	stx	$d011
	jsr	$fda3
	jsr	clone_fd50
	jsr	clone_fd15
	jsr	$e453
	jsr	$e3bf
cs_lp1:	
	lda	$d011	
	bpl	cs_lp1
cs_lp2:
	lda	$d011
	bmi	cs_lp2
	jsr	$e5a0
	lda	#$00
	sta	$d020
	sta	$d011
	jsr	$e51b
	jsr	$ff5e
	jsr	clone_f951
	
	lda	#$1b
	sta	$d011
	lda	#14
	sta	$d020
; done

	ldx	#$01		; default reg D banking
; FALL THRU
warm_start:
; we enter here with the old reg E bank in the X register.
; and SAC #$EE
	sei
	sac	#$00
	stx	store_reg_d

	ldx	#40-1
ws_lp1:
	lda	$0400,x
	sta	line_store,x
	lda	$d800,x
	sta	col_store,x
	dex
	bpl	ws_lp1
	
	lda	cfg_dtvtrans_port
	eor	#$ff
	jsr	server

	ldx	#40-1
ws_lp2:
	lda	line_store,x
	sta	$0400,x
	lda	col_store,x
	sta	$d800,x
	dex
	bpl	ws_lp2

	ldx	#trampoline_end-trampoline-1
mn_lp1:
	lda	trampoline,x
	sta	trampoline_run,x
	dex
	bpl	mn_lp1
	ldx	store_reg_d
	jmp	trampoline_run

store_reg_8:
	dc.b	0
store_reg_d:
	dc.b	0
store_reg_e:
	dc.b	0
store_d020:
	dc.b	0
trampoline:
	rorg	$100
trampoline_run:
	sac	$dd
	txa
	sac	$00
	cli
	jmp	$a474
	rend
trampoline_end:
	

;**************************************************************************
;*
;* NAME  clone_fd50
;*
;* DESCRIPTION
;*   Clone the memory init at $fd50.
;*
;******
clone_fd50:
	lda	#0
	tay
cfd50_lp1:
	sta	$0002,y
	sta	$0200,y
	sta	$0300,y
	iny
	bne	cfd50_lp1

	ldx	#$03
	lda	#$3c
	sta	$b2
	stx	$b3

	ldx	#$00
	ldy	#$a0
	jsr	$fd8c	;Set MemBounds
	rts

;**************************************************************************
;*
;* NAME  clone_fd15
;*
;* DESCRIPTION
;*   Clone the vector setup at $fd15.
;*
;******
clone_fd15:
	ldy	#$1f
cfd15_lp1:
	lda	$fd30,y
	sta	$0314,y
	dey
	bpl	cfd15_lp1
	rts


;**************************************************************************
;*
;* NAME  clone_f951
;*
;* DESCRIPTION
;*   Clone of the DTV video init at $f951.
;*
;******
clone_f951:
	ldx	#$01
	stx	$d03f

; is the config all $ff's?
	ldx	#5
	lda	#$ff
cri_lp1:
	and	cfg_hard_video,x
	dex
	bpl	cri_lp1
	cmp	#$ff
	bne	clone_f951_hard	; no, skip to hard video
; yes, do normal config

	lda	#$00
	sta	$dd03
	lda	$dd01
	sta	$aa
	and	#$03
	sta	$d040
	lda	$aa
	lsr
	lsr
	and	#$0f
	sta	$d04f
	bit	$aa
	bvs	cri_skp1
	lda	#7
	dc.b	$2c
cri_skp1:
	lda	#6
	sta	$d04e
	bit	$aa
	bmi	cri_skp2
	ldy	#$02
	dc.b	$2c
cri_skp2:
	ldy	#$05
	lda	$01
	and	#$10
	bne	cri_skp3
	tya
	clc
	adc	#$06
	tay
	lda	#$00
	sta	$d04e
cri_skp3:
	ldx	#$2
cri_lp2:
	lda	consts,y
	sta	$d041,x
	dey
	dex
	bpl	cri_lp2
cri_ex1:
	ldx	#$00
	stx	$d03f
	rts
consts:
	dc.b	$1c,$13,$2a
	dc.b	$24,$31,$5b
	dc.b	$1c,$00,$00
	dc.b	$24,$00,$00

clone_f951_hard:
	ldx	#3
cfh_lp1:
	lda	cfg_hard_video,x
	sta	$d040,x
	dex
	bpl	cfh_lp1
	ldx	#1
cfh_lp2:
	lda	cfg_hard_video+4,x
	sta	$d04e,x
	dex
	bpl	cfh_lp2
	bra	cri_ex1
	
;**************************************************************************
;*
;* external code
;*
;******
	include	"server.asm"

;**************************************************************************
;*
;* bss data
;*
;******
	seg.u	bss
line_store:
	ds.b	40
col_store:
	ds.b	40	

	echo	"range",$4000,.	
; eof
