/*************************************************************************
 *
 * FILE  boot.cpp
 * Copyright (c) 2006 Daniel Kahlin <daniel@kahlin.net>
 * Written by Daniel Kahlin <daniel@kahlin.net>
 * $Id: boot.cpp,v 1.1 2006-09-06 14:36:09 tlr Exp $
 *
 * DESCRIPTION 
 *   boot protocol transfer code
 *
 ******/

#include "boot.hpp"

Boot::Boot(Port& pt)
    : pt_(pt)
{
}

void Boot::send_file(uint16_t start, uint16_t end, uint8_t* data)
{
    send_byte_(start >> 8);
    send_byte_(start & 0xff);
    send_byte_(end >> 8);
    send_byte_(end & 0xff);

    uint8_t ck=0;
    for (int i=start; i<=end; ++i) {
	send_byte_(*data);
	ck+=(*data)+1;
	++data;
    }
    send_byte_(ck);
}
    
void Boot::send_byte_(uint8_t data)
{
    uint8_t tmp;
    // bit7-6 to d1/d0, clk=0
    tmp=(data >> 6) & 0x03;
    tmp|=0x10;  // keep ACK high to be able to read it.
    pt_.write(tmp);
    while( pt_.read()&0x10 ); // wait for ACK low

    // bit5-4 to d1/d0, clk=1
    tmp=(data >> 4) & 0x03;
    tmp|=0x08;
    tmp|=0x10;  // keep ACK high to be able to read it.
    pt_.write(tmp);
    while( !(pt_.read()&0x10) );  // wait for ACK high

    // bit3-2 to d1/d0, clk=0
    tmp=(data >> 2) & 0x03;
    tmp|=0x10;  // keep ACK high to be able to read it.
    pt_.write(tmp);
    while( pt_.read()&0x10 );  // wait for ACK low

    // bit1-0 to d1/d0, clk=1
    tmp=(data >> 0) & 0x03;
    tmp|=0x08;
    tmp|=0x10;  // keep ACK high to be able to read it.
    pt_.write(tmp);
    while( !(pt_.read()&0x10) );  // wait for ACK high
}
/* eof */
